@@ -1,4 +1,4 @@
-// this file is the javascript backing for the UI wrangling which happens in
+// this file is the JavaScript backing for the UI wrangling which happens in
 // browser.xul. It also initializes the Greasemonkey singleton which contains
 // all the main injection logic, though that should probably be a proper XPCOM
 // service and wouldn't need to be initialized in that case.
@@ -11,11 +11,12 @@ var GM_BrowserUI = new Object();
 GM_BrowserUI.QueryInterface = function(aIID) {
   if (!aIID.equals(Components.interfaces.nsISupports) &&
       !aIID.equals(Components.interfaces.gmIBrowserWindow) &&
-      !aIID.equals(Components.interfaces.nsISupportsWeakReference))
+      !aIID.equals(Components.interfaces.nsISupportsWeakReference) &&
+      !aIID.equals(Components.interfaces.nsIWebProgressListener))
     throw Components.results.NS_ERROR_NO_INTERFACE;
 
   return this;
-}
+};
 
 
 /**
@@ -27,11 +28,9 @@ GM_BrowserUI.init = function() {
   this.menuCommanders = [];
   this.currentMenuCommander = null;
 
-  GM_updateVersion();
-
   GM_listen(window, "load", GM_hitch(this, "chromeLoad"));
   GM_listen(window, "unload", GM_hitch(this, "chromeUnload"));
-}
+};
 
 /**
  * The browser XUL has loaded. Find the elements we need and set up our
@@ -46,6 +45,7 @@ GM_BrowserUI.chromeLoad = function(e) {
   this.statusLabel = document.getElementById("gm-status-label");
   this.statusPopup = document.getElementById("gm-status-popup");
   this.statusEnabledItem = document.getElementById("gm-status-enabled-item");
+  this.generalMenuEnabledItem = document.getElementById("gm-general-menu-enabled-item");
   this.toolsMenu = document.getElementById("menu_ToolsPopup");
   this.bundle = document.getElementById("gm-browser-bundle");
 
@@ -54,6 +54,11 @@ GM_BrowserUI.chromeLoad = function(e) {
     this.toolsMenu = document.getElementById("taskPopup");
   }
 
+  // songbird compat
+  if (!this.appContent && this.tabBrowser) {
+    this.appContent = this.tabBrowser.parentNode;
+  }
+
   // update visual status when enabled state changes
   this.enabledWatcher = GM_hitch(this, "refreshStatus");
   GM_prefRoot.watch("enabled", this.enabledWatcher);
@@ -73,7 +78,7 @@ GM_BrowserUI.chromeLoad = function(e) {
                           .getService(Components.interfaces.nsIWindowWatcher);
 
   // this gives us onLocationChange
-  document.getElementById("content").addProgressListener(this,
+  this.tabBrowser.addProgressListener(this,
     Components.interfaces.nsIWebProgress.NOTIFY_LOCATION);
 
   // update enabled icon
@@ -84,7 +89,7 @@ GM_BrowserUI.chromeLoad = function(e) {
                          .getService(Components.interfaces.gmIGreasemonkeyService);
 
   this.gmSvc.registerBrowser(this);
-}
+};
 
 /**
  * gmIBrowserWindow.registerMenuCommand
@@ -99,23 +104,24 @@ GM_BrowserUI.registerMenuCommand = function(menuCommand) {
                                   menuCommand.accelModifiers,
                                   menuCommand.accessKey);
   }
-}
+};
 
 /**
  * gmIBrowserWindow.openInTab
  */
 GM_BrowserUI.openInTab = function(domWindow, url) {
   if (this.isMyWindow(domWindow)) {
-    document.getElementById("content").addTab(url);
+    this.tabBrowser.addTab(url);
   }
-}
+};
 
 /**
  * Gets called when a DOMContentLoaded event occurs somewhere in the browser.
- * If that document is in in the top-level window of the focused tab, find 
+ * If that document is in in the top-level window of the focused tab, find
  * it's menu items and activate them.
  */
 GM_BrowserUI.contentLoad = function(e) {
+  var safeWin;
   var unsafeWin;
   var href;
   var commander;
@@ -124,54 +130,47 @@ GM_BrowserUI.contentLoad = function(e) {
     return;
   }
 
-  if (GM_deepWrappersEnabled(window)) {
-    // when deep wrappers are enabled, e.target is already a deep xpcnw
-    unsafeWin = e.target.defaultView;
-
-    // in DPa2, there was a bug that made this *not* a deep wrapper.
-    if (unsafeWin.wrappedJSObject) {
-      unsafeWin = unsafeWin.wrappedJSObject;
-    }
-
-    href = e.target.location.href;
-  } else {
-    // otherwise we need to wrap it manually
-    unsafeWin = new XPCNativeWrapper(
-                  new XPCNativeWrapper(e, "target").target,
-                  "defaultView").defaultView;
-    href = new XPCNativeWrapper(
-              new XPCNativeWrapper(unsafeWin, "location").location,
-              "href").href;
-  }
+  safeWin = e.target.defaultView;
+  unsafeWin = safeWin.wrappedJSObject;
+  href = safeWin.location.href;
 
   if (GM_isGreasemonkeyable(href)) {
     commander = this.getCommander(unsafeWin);
 
-    // if this content load is in the focused tab, attach the menuCommaander  
+    // if this content load is in the focused tab, attach the menuCommaander
     if (unsafeWin == this.tabBrowser.selectedBrowser.contentWindow) {
       this.currentMenuCommander = commander;
       this.currentMenuCommander.attach();
     }
 
-    this.gmSvc.domContentLoaded({ wrappedJSObject: unsafeWin });
-  
+    this.gmSvc.domContentLoaded({ wrappedJSObject: unsafeWin }, window);
+
     GM_listen(unsafeWin, "pagehide", GM_hitch(this, "contentUnload"));
   }
 
-  if (!href.match(/\.user\.js$/)) {
-    return;
+  // Show the greasemonkey install banner if we are navigating to a .user.js
+  // file in a top-level tab.
+  if (href.match(/\.user\.js$/) && safeWin == safeWin.top) {
+    var browser = this.tabBrowser.getBrowserForDocument(safeWin.document);
+    this.showInstallBanner(browser);
   }
+};
 
-  var browser = this.tabBrowser.selectedBrowser;
+
+/**
+ * Shows the install banner across the top of the tab that is displayed when
+ * a user selects "show script source" in the install dialog.
+ */
+GM_BrowserUI.showInstallBanner = function(browser) {
   var greeting = this.bundle.getString("greeting.msg");
 
   if (this.tabBrowser.showMessage) {
     // Firefox 1.5 and lower
     this.tabBrowser.showMessage(
       browser,
-      "chrome://greasemonkey/content/status_on.gif",
+      "chrome://greasemonkey/content/icon_small.png",
       greeting,
-      this.bundle.getString('greeting.btn'),
+      this.bundle.getString("greeting.btn"),
       null /* default doc shell */,
       "install-userscript",
       null /* no popuup */,
@@ -181,34 +180,37 @@ GM_BrowserUI.contentLoad = function(e) {
   } else {
     // Firefox 2.0+
     var notificationBox = this.tabBrowser.getNotificationBox(browser);
-  
+
     // Remove existing notifications. Notifications get removed
     // automatically onclick and on page navigation, but we need to remove
     // them ourselves in the case of reload, or they stack up.
     for (var i = 0, child; child = notificationBox.childNodes[i]; i++) {
       if (child.getAttribute("value") == "install-userscript") {
-	notificationBox.removeNotification(child);
+        notificationBox.removeNotification(child);
       }
     }
 
     var notification = notificationBox.appendNotification(
       greeting,
       "install-userscript",
-      "chrome://greasemonkey/content/status_on.gif",
+      "chrome://greasemonkey/content/icon_small.png",
       notificationBox.PRIORITY_WARNING_MEDIUM,
-      [{ label: this.bundle.getString('greeting.btn'),
-	 accessKey: "I",
-	 popup: null,
-	 callback: GM_hitch(this, "installCurrentScript") }]);
+      [{
+        label: this.bundle.getString("greeting.btn"),
+        accessKey: "I",
+        popup: null,
+        callback: GM_hitch(this, "installCurrentScript")
+      }]
+    );
   }
 };
 
 /**
- * Called from greasemonkey service when we should load a user script. 
+ * Called from greasemonkey service when we should load a user script.
  */
 GM_BrowserUI.startInstallScript = function(uri, timer) {
   if (!timer) {
-    // docs for nsicontentpolicy say we're not supposed to block, so short 
+    // docs for nsicontentpolicy say we're not supposed to block, so short
     // timer.
     window.setTimeout(
       function() { GM_BrowserUI.startInstallScript(uri, true) }, 0);
@@ -228,15 +230,10 @@ GM_BrowserUI.startInstallScript = function(uri, timer) {
 GM_BrowserUI.showScriptView = function(scriptDownloader) {
   this.scriptDownloader_ = scriptDownloader;
 
-  var ioSvc = Components.classes["@mozilla.org/network/io-service;1"]
-                        .getService(Components.interfaces.nsIIOService);
-  var uri = ioSvc.newFileURI(scriptDownloader.script.file);
-
-  var tab = this.tabBrowser.addTab(uri.spec);
+  var tab = this.tabBrowser.addTab(scriptDownloader.script.previewURL);
   var browser = this.tabBrowser.getBrowserForTab(tab);
 
   this.tabBrowser.selectedTab = tab;
-
 };
 
 /**
@@ -257,12 +254,13 @@ GM_BrowserUI.observe = function(subject, topic, data) {
  * Handles the install button getting clicked.
  */
 GM_BrowserUI.installCurrentScript = function() {
-  var config = new Config();
-  config.load();
-  config.install(this.scriptDownloader_.script);
-  this.showHorrayMessage(this.scriptDownloader_.script.name);
+  this.scriptDownloader_.installScript();
 };
 
+GM_BrowserUI.installScript = function(script){
+  GM_getConfig().install(script);
+  this.showHorrayMessage(script.name);
+};
 
 /**
  * The browser's location has changed. Usually, we don't care. But in the case
@@ -277,16 +275,16 @@ GM_BrowserUI.onLocationChange = function(a,b,c) {
 
   var menuCommander = this.getCommander(this.tabBrowser.selectedBrowser.
                                         contentWindow);
-  
+
   if (menuCommander) {
     this.currentMenuCommander = menuCommander;
     this.currentMenuCommander.attach();
   }
-}
+};
 
 /**
- * A content document has unloaded. We need to remove it's menuCommander to 
- * avoid leaking it's memory. 
+ * A content document has unloaded. We need to remove it's menuCommander to
+ * avoid leaking it's memory.
  */
 GM_BrowserUI.contentUnload = function(e) {
   if (e.persisted) {
@@ -295,29 +293,25 @@ GM_BrowserUI.contentUnload = function(e) {
 
   var unsafeWin = e.target.defaultView;
 
-  // remove the commander for this document  
+  // remove the commander for this document
   var commander = null;
-  
+
   // looping over commanders rather than using getCommander because we need
   // the index into commanders.splice.
   for (var i = 0; item = this.menuCommanders[i]; i++) {
     if (item.win == unsafeWin) {
 
-      log("* Found corresponding commander. Is currentMenuCommander: " + 
-          (item.commander == this.currentMenuCommander));
-
       if (item.commander == this.currentMenuCommander) {
         this.currentMenuCommander.detach();
         this.currentMenuCommander = null;
       }
-      
+
       this.menuCommanders.splice(i, 1);
 
-      log("* Found and removed corresponding commander")
       break;
     }
   }
-}
+};
 
 /**
  * The browser XUL has unloaded. We need to let go of the pref watcher so
@@ -330,7 +324,7 @@ GM_BrowserUI.chromeUnload = function() {
   this.tabBrowser.removeProgressListener(this);
   this.gmSvc.unregisterBrowser(this);
   delete this.menuCommanders;
-}
+};
 
 /**
  * Called when the content area context menu is showing. We figure out whether
@@ -349,7 +343,7 @@ GM_BrowserUI.contextMenuShowing = function() {
   contextItem.hidden =
     contextSep.hidden =
     !this.getUserScriptLinkUnderPointer();
-}
+};
 
 
 GM_BrowserUI.getUserScriptLinkUnderPointer = function() {
@@ -369,23 +363,25 @@ GM_BrowserUI.getUserScriptLinkUnderPointer = function() {
   var uri = ioSvc.newURI(culprit.href, null, null);
 
   return uri;
-}
+};
 
 GM_BrowserUI.toolsMenuShowing = function() {
   var installItem = ge("userscript-tools-install");
-  var collapsed = true;
+  var hidden = true;
 
   if (window._content && window._content.location &&
       window.content.location.href.match(/\.user\.js(\?|$)/i)) {
-    collapsed = false;
+    hidden = false;
   }
 
-  installItem.setAttribute("collapsed", collapsed.toString());
-}
+  // Better to use hidden than collapsed because collapsed still allows you to
+  // select the item using keyboard navigation, but hidden doesn't.
+  installItem.setAttribute("hidden", hidden.toString());
+};
 
 
 /**
- * Helper method which gets the menuCommander corresponding to a given 
+ * Helper method which gets the menuCommander corresponding to a given
  * document
  */
 GM_BrowserUI.getCommander = function(unsafeWin) {
@@ -400,13 +396,13 @@ GM_BrowserUI.getCommander = function(unsafeWin) {
   this.menuCommanders.push({win:unsafeWin, commander:commander});
 
   return commander;
-}
+};
 
 /**
  * Helper to determine if a given dom window is in this tabbrowser
  */
 GM_BrowserUI.isMyWindow = function(domWindow) {
-  var tabbrowser = getBrowser();  
+  var tabbrowser = getBrowser();
   var browser;
 
   for (var i = 0; browser = tabbrowser.browsers[i]; i++) {
@@ -416,63 +412,114 @@ GM_BrowserUI.isMyWindow = function(domWindow) {
   }
 
   return false;
+};
+
+function GM_showGeneralPopup(aEvent) {
+  // set the enabled/disabled state
+  GM_BrowserUI.generalMenuEnabledItem.setAttribute("checked", GM_getEnabled());
 }
 
 function GM_showPopup(aEvent) {
-  var config = new Config(getScriptFile("config.xml"));
-  config.load();
+  function urlsOfAllFrames(contentWindow) {
+    function collect(contentWindow) {
+      urls = urls.concat(urlsOfAllFrames(contentWindow));
+    }
+    var urls = [contentWindow.location.href];
+    Array.prototype.slice.call(contentWindow.frames).forEach(collect);
+    return urls;
+  }
+
+  function uniq(a) {
+    var seen = {}, list = [], item;
+    for (var i = 0; i < a.length; i++) {
+      item = a[i];
+      if (!seen.hasOwnProperty(item))
+        seen[item] = list.push(item);
+    }
+    return list;
+  }
+
+  function scriptsMatching(urls) {
+
+    function testMatchURLs(script) {
+
+      function testMatchURL(url) {
+        return script.matchesURL(url);
+      }
+
+      return urls.some(testMatchURL);
+    }
+
+    return GM_getConfig().getMatchingScripts(testMatchURLs);
+  }
+
+  function appendScriptToPopup(script) {
+    var mi = document.createElement("menuitem");
+    mi.setAttribute("label", script.name);
+    mi.script = script;
+    mi.setAttribute("type", "checkbox");
+    mi.setAttribute("checked", script.enabled.toString());
+    popup.insertBefore(mi, tail);
+  }
+
   var popup = aEvent.target;
-  var url = getBrowser().contentWindow.document.location.href;
+  var tail = document.getElementById("gm-status-no-scripts-sep");
 
   // set the enabled/disabled state
   GM_BrowserUI.statusEnabledItem.setAttribute("checked", GM_getEnabled());
 
   // remove all the scripts from the list
   for (var i = popup.childNodes.length - 1; i >= 0; i--) {
-    if (popup.childNodes[i].hasAttribute("value")) {
+    if (popup.childNodes[i].script) {
       popup.removeChild(popup.childNodes[i]);
     }
   }
 
-  var foundInjectedScript = false;
+  var urls = uniq( urlsOfAllFrames( getBrowser().contentWindow ));
+  var runsOnTop = scriptsMatching( [urls.shift()] ); // first url = top window
+  var runsFramed = scriptsMatching( urls ); // remainder are all its subframes
 
-  // build the new list of scripts
-  for (var i = 0, script = null; script = config.scripts[i]; i++) {
-    incloop: for (var j = 0; j < script.includes.length; j++) {
-      var pattern = convert2RegExp(script.includes[j]);
-      if (pattern.test(url)) {
-        for (var k = 0; k < script.excludes.length; k++) {
-          pattern = convert2RegExp(script.excludes[k]);
-          if (pattern.test(url)) {
-            break incloop;
-          }
-        }
-
-        foundInjectedScript = true;
-
-        var mi = document.createElement('menuitem');
-        mi.setAttribute('label', script.name);
-        mi.setAttribute('value', i);
-        mi.setAttribute('type', 'checkbox');
-        mi.setAttribute('checked', script.enabled.toString());
-
-        popup.insertBefore(mi, document.getElementById("gm-status-no-scripts-sep"));
-
-        break incloop;
+  // drop all runsFramed scripts already present in runsOnTop
+  for (var i = 0; i < runsOnTop.length; i++) {
+    var j = 0, item = runsOnTop[i];
+    while (j < runsFramed.length) {
+      if (item === runsFramed[j]) {
+        runsFramed.splice(j, 1);
+      } else {
+        j++;
       }
     }
   }
 
+  // build the new list of scripts
+  if (runsFramed.length) {
+    runsFramed.forEach(appendScriptToPopup);
+    var separator = document.createElement("menuseparator");
+    separator.setAttribute("value", "hack"); // to get removed in the loop above
+    popup.insertBefore(separator, tail);
+  }
+  runsOnTop.forEach(appendScriptToPopup);
+
+  var foundInjectedScript = !!(runsFramed.length + runsOnTop.length);
   document.getElementById("gm-status-no-scripts").collapsed = foundInjectedScript;
 }
 
+/**
+ * Handle clicking one of the items in the popup. Left-click toggles the enabled
+ * state, rihgt-click opens in an editor.
+ */
 function GM_popupClicked(aEvent) {
-  var config = new Config(getScriptFile("config.xml"));
-  config.load();
-  var scriptNum=aEvent.target.value;
-  if (!config.scripts[scriptNum]) return;
-  config.scripts[scriptNum].enabled=!config.scripts[scriptNum].enabled;
-  config.save();
+  if (aEvent.button == 0 || aEvent.button == 2) {
+    var script = aEvent.target.script;
+    if (!script) return;
+
+    if (aEvent.button == 0) // left-click: toggle enabled state
+      script.enabled =! script.enabled;
+    else // right-click: open in editor
+      openInEditor(script);
+
+    closeMenus(aEvent.target);
+  }
 }
 
 /**
@@ -482,39 +529,25 @@ function GM_popupClicked(aEvent) {
  */
 GM_BrowserUI.refreshStatus = function() {
   if (GM_getEnabled()) {
-    this.statusImage.src = "chrome://greasemonkey/content/status_on.gif";
-    this.statusImage.tooltipText = this.bundle.getString('tooltip.enabled');
+    this.statusImage.src = "chrome://greasemonkey/content/icon_small.png";
+    this.statusImage.tooltipText = this.bundle.getString("tooltip.enabled");
   } else {
-    this.statusImage.src = "chrome://greasemonkey/content/status_off.gif";
-    this.statusImage.tooltipText = this.bundle.getString('tooltip.disabled');
+    this.statusImage.src = "chrome://greasemonkey/content/icon_small_disabled.png";
+    this.statusImage.tooltipText = this.bundle.getString("tooltip.disabled");
   }
 
   this.statusImage.style.opacity = "1.0";
-}
+};
 
 GM_BrowserUI.newUserScript = function() {
-  var tempname = "newscript.user.js";
-  
-  var source = getContentDir();
-  source.append("template.user.js");
-  
-  var dest = Components.classes["@mozilla.org/file/directory_service;1"]
-                       .getService(Components.interfaces.nsIProperties)
-                       .get("TmpD", Components.interfaces.nsILocalFile);
-        
-  var destFile = dest.clone().QueryInterface(Components.interfaces.nsILocalFile);
-  destFile.append(tempname);
-  
-  if (destFile.exists()) {
-    destFile.remove(false);
-  }
-
-  source.copyTo(dest, tempname);
-
-  openInEditor(
-    destFile,
-    this.bundle.getString("editor.prompt"));
-}
+  var windowWatcher = Components
+    .classes["@mozilla.org/embedcomp/window-watcher;1"]
+    .getService(Components.interfaces.nsIWindowWatcher);
+  windowWatcher.openWindow(
+    window, "chrome://greasemonkey/content/newscript.xul", null,
+    "chrome,dependent,centerscreen,resizable,dialog", null
+  );
+};
 
 GM_BrowserUI.showStatus = function(message, autoHide) {
   if (this.statusLabel.collapsed) {
@@ -537,11 +570,11 @@ GM_BrowserUI.showStatus = function(message, autoHide) {
   this.statusLabel.value = message;
   var max = label.boxObject.width;
 
-  this.showAnimation = new Accelimation(this.statusLabel.style, 
+  this.showAnimation = new Accelimation(this.statusLabel.style,
                                           "width", max, 300, 2, "px");
   this.showAnimation.onend = GM_hitch(this, "showStatusAnimationEnd", autoHide);
   this.showAnimation.start();
-}
+};
 
 GM_BrowserUI.showStatusAnimationEnd = function(autoHide) {
   this.showAnimation = null;
@@ -549,7 +582,7 @@ GM_BrowserUI.showStatusAnimationEnd = function(autoHide) {
   if (autoHide) {
     this.setAutoHideTimer();
   }
-}
+};
 
 GM_BrowserUI.setAutoHideTimer = function() {
   if (this.autoHideTimer) {
@@ -557,52 +590,72 @@ GM_BrowserUI.setAutoHideTimer = function() {
   }
 
   this.autoHideTimer = window.setTimeout(GM_hitch(this, "hideStatus"), 3000);
-}
+};
+
+GM_BrowserUI.hideStatusImmediately = function() {
+  if (this.showAnimation) {
+    this.showAnimation.stop();
+    this.showAnimation = null;
+  }
+
+  if (this.hideAnimation) {
+    this.hideAnimation.stop();
+    this.hideAnimation = null;
+  }
+
+  if (this.autoHideTimer) {
+    window.clearTimeout(this.autoHideTimer);
+    this.autoHideTimer = null;
+  }
+
+  this.statusLabel.style.width = "0";
+  this.statusLabel.collapsed = true;
+};
 
 GM_BrowserUI.hideStatus = function() {
   if (!this.hideAnimation) {
     this.autoHideTimer = null;
-    this.hideAnimation = new Accelimation(this.statusLabel.style, 
+    this.hideAnimation = new Accelimation(this.statusLabel.style,
                                             "width", 0, 300, 2, "px");
     this.hideAnimation.onend = GM_hitch(this, "hideStatusAnimationEnd");
     this.hideAnimation.start();
   }
-}
+};
 
 GM_BrowserUI.hideStatusAnimationEnd = function() {
   this.hideAnimation = null;
   this.statusLabel.collapsed = true;
-}
+};
 
 // necessary for webProgressListener implementation
-GM_BrowserUI.onProgressChange = function(webProgress,b,c,d,e,f){}
-GM_BrowserUI.onStateChange = function(a,b,c,d){}
-GM_BrowserUI.onStatusChange = function(a,b,c,d){}
-GM_BrowserUI.onSecurityChange = function(a,b,c){}
-GM_BrowserUI.onLinkIconAvailable = function(a){}
+GM_BrowserUI.onProgressChange = function(webProgress,b,c,d,e,f){};
+GM_BrowserUI.onStateChange = function(a,b,c,d){};
+GM_BrowserUI.onStatusChange = function(a,b,c,d){};
+GM_BrowserUI.onSecurityChange = function(a,b,c){};
+GM_BrowserUI.onLinkIconAvailable = function(a){};
 
 GM_BrowserUI.showHorrayMessage = function(scriptName) {
   this.showStatus("'" + scriptName + "' " + this.bundle.getString("statusbar.installed"), true);
-}
+};
 
 GM_BrowserUI.installMenuItemClicked = function() {
   GM_BrowserUI.startInstallScript(
-    GM_BrowserUI.getUserScriptLinkUnderPointer());
-}
+    gBrowser.currentURI
+  );
+};
 
 GM_BrowserUI.viewContextItemClicked = function() {
   var uri = GM_BrowserUI.getUserScriptLinkUnderPointer();
 
   this.scriptDownloader_ = new ScriptDownloader(window, uri, this.bundle);
   this.scriptDownloader_.startViewScript();
-}
+};
 
 GM_BrowserUI.manageMenuItemClicked = function() {
-   window.openDialog("chrome://greasemonkey/content/manage.xul", "manager", 
-    "resizable,centerscreen,modal");
-}
+   GM_openUserScriptManager();
+};
 
 //loggify(GM_BrowserUI, "GM_BrowserUI");
 
-log("calling init...")
+log("calling init...");
 GM_BrowserUI.init();
