@@ -8,15 +8,49 @@ const Ci = Components.interfaces;
 const appSvc = Cc["@mozilla.org/appshell/appShellService;1"]
                  .getService(Ci.nsIAppShellService);
 
+const gmSvcFilename = Components.stack.filename;
+
 function alert(msg) {
   Cc["@mozilla.org/embedcomp/prompt-service;1"]
     .getService(Ci.nsIPromptService)
-    .alert(null, "message from your mom", msg);
+    .alert(null, "Greasemonkey alert", msg);
+}
+
+// Examines the stack to determine if an API should be callable.
+function GM_apiLeakCheck(apiName) {
+  var stack = Components.stack;
+
+  do {
+    // Valid stack frames for GM api calls are: native and js when coming from
+    // chrome:// URLs and the greasemonkey.js component's file:// URL.
+    if (2 == stack.language) {
+      // NOTE: In FF 2.0.0.0, I saw that stack.filename can be null for JS/XPCOM
+      // services. This didn't happen in FF 2.0.0.11; I'm not sure when it
+      // changed.
+      if (stack.filename != null &&
+          stack.filename != gmSvcFilename &&
+          stack.filename.substr(0, 6) != "chrome") {
+        GM_logError(new Error("Greasemonkey access violation: unsafeWindow " +
+                    "cannot call " + apiName + "."));
+        return false;
+      }
+    }
+
+    stack = stack.caller;
+  } while (stack);
+
+  return true;
 }
 
 var greasemonkeyService = {
-
+  _config: null,
+  get config() {
+    if (!this._config)
+      this._config = new Config();
+    return this._config;
+  },
   browserWindows: [],
+  updater: null,
 
 
   // nsISupports
@@ -26,7 +60,7 @@ var greasemonkeyService = {
         !aIID.equals(Ci.nsISupportsWeakReference) &&
         !aIID.equals(Ci.gmIGreasemonkeyService) &&
         !aIID.equals(Ci.nsIWindowMediatorListener) &&
-	!aIID.equals(Ci.nsIContentPolicy)) {
+        !aIID.equals(Ci.nsIContentPolicy)) {
       throw Components.results.NS_ERROR_NO_INTERFACE;
     }
 
@@ -48,6 +82,7 @@ var greasemonkeyService = {
 
     for (var i = 0; existing = this.browserWindows[i]; i++) {
       if (existing == browserWin) {
+        // NOTE: Unlocalised strings
         throw new Error("Browser window has already been registered.");
       }
     }
@@ -68,14 +103,23 @@ var greasemonkeyService = {
     throw new Error("Browser window is not registered.");
   },
 
-  domContentLoaded: function(wrappedWindow) {
-    var unsafeWin = wrappedWindow.wrappedJSObject;
+  domContentLoaded: function(wrappedContentWin, chromeWin) {
+    var unsafeWin = wrappedContentWin.wrappedJSObject;
     var unsafeLoc = new XPCNativeWrapper(unsafeWin, "location").location;
     var href = new XPCNativeWrapper(unsafeLoc, "href").href;
     var scripts = this.initScripts(href);
 
     if (scripts.length > 0) {
-      this.injectScripts(scripts, href, unsafeWin);
+      this.injectScripts(scripts, href, unsafeWin, chromeWin);
+    }
+
+    // Need to wait until well after startup for prefs store and extension
+    // manager to be initialized. First page load is a convenient place.
+    if (!this.updater) {
+      // Note: the param to this has to match the extension ID in install.rdf
+      this.updater = new ExtensionUpdater(
+          "{e4a8a97b-f2ed-450b-b12d-ee082ba24781}");
+      this.updater.updatePeriodically();
     }
   },
 
@@ -109,28 +153,50 @@ var greasemonkeyService = {
       .getService(Ci.mozIJSSubScriptLoader)
       .loadSubScript("chrome://greasemonkey/content/xmlhttprequester.js");
 
+    Cc["@mozilla.org/moz/jssubscript-loader;1"]
+      .getService(Ci.mozIJSSubScriptLoader)
+      .loadSubScript("chrome://greasemonkey/content/updater.js");
+
     //loggify(this, "GM_GreasemonkeyService");
   },
 
   shouldLoad: function(ct, cl, org, ctx, mt, ext) {
     var ret = Ci.nsIContentPolicy.ACCEPT;
 
-    if (ct == Ci.nsIContentPolicy.TYPE_DOCUMENT && 
-	cl.spec.match(/\.user\.js$/)) {
+    // block content detection of greasemonkey by denying GM
+    // chrome content, unless loaded from chrome
+    if (org && org.scheme != "chrome" && cl.scheme == "chrome" &&
+        cl.host == "greasemonkey") {
+      return Ci.nsIContentPolicy.REJECT_SERVER;
+    }
+
+    // don't intercept anything when GM is not enabled
+    if (!GM_getEnabled()) {
+      return ret;
+    }
+
+    // don't interrupt the view-source: scheme
+    // (triggered if the link in the error console is clicked)
+    if ("view-source" == cl.scheme) {
+      return ret;
+    }
+
+    if (ct == Ci.nsIContentPolicy.TYPE_DOCUMENT &&
+        cl.spec.match(/\.user\.js$/)) {
 
       dump("shouldload: " + cl.spec + "\n");
       dump("ignorescript: " + this.ignoreNextScript_ + "\n");
 
       if (!this.ignoreNextScript_) {
-	if (!this.isTempScript(cl)) {
-	  var winWat = Cc["@mozilla.org/embedcomp/window-watcher;1"]
-	    .getService(Ci.nsIWindowWatcher);
+        if (!this.isTempScript(cl)) {
+          var winWat = Cc["@mozilla.org/embedcomp/window-watcher;1"]
+            .getService(Ci.nsIWindowWatcher);
 
-	  if (winWat.activeWindow && winWat.activeWindow.GM_BrowserUI) {
-	    winWat.activeWindow.GM_BrowserUI.startInstallScript(cl);
-	    ret = Ci.nsIContentPolicy.REJECT_REQUEST;
-	  }
-	}
+          if (winWat.activeWindow && winWat.activeWindow.GM_BrowserUI) {
+            winWat.activeWindow.GM_BrowserUI.startInstallScript(cl);
+            ret = Ci.nsIContentPolicy.REJECT_REQUEST;
+          }
+        }
       }
     }
 
@@ -138,6 +204,10 @@ var greasemonkeyService = {
     return ret;
   },
 
+  shouldProcess: function(ct, cl, org, ctx, mt, ext) {
+    return Ci.nsIContentPolicy.ACCEPT;
+  },
+
   ignoreNextScript: function() {
     dump("ignoring next script...\n");
     this.ignoreNextScript_ = true;
@@ -160,54 +230,38 @@ var greasemonkeyService = {
   },
 
   initScripts: function(url) {
-    var config = new Config(getScriptFile("config.xml"));
-    var scripts = [];
-    config.load();
-    
-    outer:
-    for (var i = 0; i < config.scripts.length; i++) {
-      var script = config.scripts[i];
-      if (script.enabled) {
-        for (var j = 0; j < script.includes.length; j++) {
-          var pattern = convert2RegExp(script.includes[j]);
-
-          if (pattern.test(url)) {
-            for (var k = 0; k < script.excludes.length; k++) {
-              pattern = convert2RegExp(script.excludes[k]);
-    
-              if (pattern.test(url)) {
-                continue outer;
-              }
-            }
-
-            scripts.push(script);
-
-            continue outer;
-          }
-        }
-      }
+    function testMatch(script) {
+      return script.enabled && script.matchesURL(url);
     }
 
-    log("* number of matching scripts: " + scripts.length);
-    return scripts;
+    return GM_getConfig().getMatchingScripts(testMatch);
   },
 
-  injectScripts: function(scripts, url, unsafeContentWin) {
+  injectScripts: function(scripts, url, unsafeContentWin, chromeWin) {
     var sandbox;
     var script;
     var logger;
+    var console;
     var storage;
     var xmlhttpRequester;
+    var resources;
     var safeWin = new XPCNativeWrapper(unsafeContentWin);
     var safeDoc = safeWin.document;
 
+    // detect and grab reference to firebug console and context, if it exists
+    var firebugConsole = this.getFirebugConsole(unsafeContentWin, chromeWin);
+
     for (var i = 0; script = scripts[i]; i++) {
       sandbox = new Components.utils.Sandbox(safeWin);
 
       logger = new GM_ScriptLogger(script);
+
+      console = firebugConsole ? firebugConsole : new GM_console(script);
+
       storage = new GM_ScriptStorage(script);
-      xmlhttpRequester = new GM_xmlhttpRequester(unsafeContentWin, 
+      xmlhttpRequester = new GM_xmlhttpRequester(unsafeContentWin,
                                                  appSvc.hiddenDOMWindow);
+      resources = new GM_Resources(script);
 
       sandbox.window = safeWin;
       sandbox.document = sandbox.window.document;
@@ -219,30 +273,48 @@ var greasemonkeyService = {
       // add our own APIs
       sandbox.GM_addStyle = function(css) { GM_addStyle(safeDoc, css) };
       sandbox.GM_log = GM_hitch(logger, "log");
+      sandbox.console = console;
       sandbox.GM_setValue = GM_hitch(storage, "setValue");
       sandbox.GM_getValue = GM_hitch(storage, "getValue");
+      sandbox.GM_getResourceURL = GM_hitch(resources, "getResourceURL");
+      sandbox.GM_getResourceText = GM_hitch(resources, "getResourceText");
       sandbox.GM_openInTab = GM_hitch(this, "openInTab", unsafeContentWin);
-      sandbox.GM_xmlhttpRequest = GM_hitch(xmlhttpRequester, 
+      sandbox.GM_xmlhttpRequest = GM_hitch(xmlhttpRequester,
                                            "contentStartRequest");
-      sandbox.GM_registerMenuCommand = GM_hitch(this, 
-                                                "registerMenuCommand", 
+      sandbox.GM_registerMenuCommand = GM_hitch(this,
+                                                "registerMenuCommand",
                                                 unsafeContentWin);
 
       sandbox.__proto__ = safeWin;
 
-      try {
-        this.evalInSandbox("(function(){\n" +
-                           getContents(getScriptFileURI(script.filename)) +
-                           "\n})()",
-                           url, 
-                           sandbox);
-      } catch (e) {
-        GM_logError(e);
-      }
+      var contents = script.textContent;
+
+      var requires = [];
+      var offsets = [];
+      var offset = 0;
+
+      script.requires.forEach(function(req){
+        var contents = req.textContent;
+        var lineCount = contents.split("\n").length;
+        requires.push(contents);
+        offset += lineCount;
+        offsets.push(offset);
+      })
+      script.offsets = offsets;
+
+      var scriptSrc = "(function(){\n" +
+                         requires.join("\n") +
+                         "\n" +
+                         contents +
+                         "\n})()";
+      this.evalInSandbox(scriptSrc,
+                         url,
+                         sandbox,
+                         script);
     }
   },
 
-  registerMenuCommand: function(unsafeContentWin, commandName, commandFunc, 
+  registerMenuCommand: function(unsafeContentWin, commandName, commandFunc,
                                 accelKey, accelModifiers, accessKey) {
     var command = {name: commandName,
                    accelKey: accelKey,
@@ -264,20 +336,90 @@ var greasemonkeyService = {
     }
   },
 
-  evalInSandbox: function(code, codebase, sandbox) {
-    // DP beta+
-    if (Components.utils && Components.utils.Sandbox) {
-      Components.utils.evalInSandbox(code, sandbox);
-    // DP alphas
-    } else if (Components.utils && Components.utils.evalInSandbox) {
-      Components.utils.evalInSandbox(code, codebase, sandbox);
-    // 1.0.x
-    } else if (Sandbox) {
-      evalInSandbox(code, sandbox, codebase);
+  evalInSandbox: function(code, codebase, sandbox, script) {
+    if (!(Components.utils && Components.utils.Sandbox)) {
+      var e = new Error("Could not create sandbox.");
+      GM_logError(e, 0, e.fileName, e.lineNumber);
     } else {
-      throw new Error("Could not create sandbox.");
+      try {
+        // workaround for https://bugzilla.mozilla.org/show_bug.cgi?id=307984
+        var lineFinder = new Error();
+        Components.utils.evalInSandbox(code, sandbox);
+      } catch (e) {
+        // try to find the line of the actual error line
+        var line = e.lineNumber;
+        if (4294967295 == line) {
+          // Line number is reported as max int in edge cases.  Sometimes
+          // the right one is in the "location", instead.  Look there.
+          if (e.location && e.location.lineNumber) {
+            line = e.location.lineNumber;
+          } else {
+            // Reporting max int is useless, if we couldn't find it in location
+            // either, forget it.  Value of 0 isn't shown in the console.
+            line = 0;
+          }
+        }
+
+        if (line) {
+          var err = this.findError(script, line - lineFinder.lineNumber - 1);
+          GM_logError(
+            e, // error obj
+            0, // 0 = error (1 = warning)
+            err.uri,
+            err.lineNumber
+          );
+        } else {
+          GM_logError(
+            e, // error obj
+            0, // 0 = error (1 = warning)
+            script.fileURL,
+            0
+          );
+        }
+      }
     }
   },
+
+  findError: function(script, lineNumber){
+    var start = 0;
+    var end = 1;
+
+    for (var i = 0; i < script.offsets.length; i++) {
+      end = script.offsets[i];
+      if (lineNumber < end) {
+        return {
+          uri: script.requires[i].fileURL,
+          lineNumber: (lineNumber - start)
+        };
+      }
+      start = end;
+    }
+
+    return {
+      uri: script.fileURL,
+      lineNumber: (lineNumber - end)
+    };
+  },
+
+  getFirebugConsole: function(unsafeContentWin, chromeWin) {
+    var firebugConsoleCtor = null;
+    var firebugContext = null;
+
+    if (chromeWin && chromeWin.FirebugConsole) {
+      firebugConsoleCtor = chromeWin.FirebugConsole;
+      firebugContext = chromeWin.top.TabWatcher
+        .getContextByWindow(unsafeContentWin);
+
+      // on first load (of multiple tabs) the context might not exist
+      if (!firebugContext) firebugConsoleCtor = null;
+    }
+
+    if (firebugConsoleCtor && firebugContext) {
+      return new firebugConsoleCtor(firebugContext, unsafeContentWin);
+    } else {
+      return null;
+    }
+  }
 };
 
 greasemonkeyService.wrappedJSObject = greasemonkeyService;
@@ -286,7 +428,7 @@ greasemonkeyService.wrappedJSObject = greasemonkeyService;
 
 
 
-/** 
+/**
  * XPCOM Registration goop
  */
 var Module = new Object();
@@ -309,28 +451,28 @@ Module.registerSelf = function(compMgr, fileSpec, location, type) {
                           true,
                           true);
 
-  catMgr.addCategoryEntry("content-policy", 
-			  CONTRACTID,
+  catMgr.addCategoryEntry("content-policy",
+                          CONTRACTID,
                           CONTRACTID,
                           true,
                           true);
-}
+};
 
 Module.getClassObject = function(compMgr, cid, iid) {
   if (!cid.equals(CID)) {
     throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
   }
-  
+
   if (!iid.equals(Ci.nsIFactory)) {
     throw Components.results.NS_ERROR_NO_INTERFACE;
   }
 
   return Factory;
-}
+};
 
 Module.canUnload = function(compMgr) {
   return true;
-}
+};
 
 
 var Factory = new Object();
@@ -341,7 +483,7 @@ Factory.createInstance = function(outer, iid) {
   }
 
   return greasemonkeyService;
-}
+};
 
 
 function NSGetModule(compMgr, fileSpec) {
